//
// RenkoBarsType - correct implementation, that includes wick data
//
// written by aslan
//
#region Using declarations
using System;
using System.Collections;
using System.ComponentModel;
using System.Text;
#endregion

// This namespace holds all bars types. Do not change it.
namespace NinjaTrader.Data
{
	public class WickedRenkoBarsType : BarsType
	{
		private static bool	registered	= Data.BarsType.Register(new WickedRenkoBarsType());

		private double		offset;
		private double		renkoHigh;
		private double		renkoLow;

		public override void Add(Data.Bars bars, double open, double high, double low, double close, DateTime time, long volume, bool isRealtime)
		{
			if ((bars.Count == 0) || (bars.IsNewSession(time))) 
			{
				int    boxSize  = bars.Period.Value;
				double tickSize = bars.Instrument.MasterInstrument.TickSize;
				offset = boxSize * tickSize;

				if (bars.Count != 0) 
				{
					// close out last bar in session and set open == close
					Data.Bar bar = (Bar) bars.Get(bars.Count-1);
					bars.RemoveLastBar();  // Note: bar is now just a local var and not in series!
					AddBar(bars, bar.Close, bar.High, bar.Low, bar.Close, bar.Time, bar.Volume, isRealtime);
				}
				
				renkoHigh = close + offset;
				renkoLow  = close - offset;
				AddBar(bars, close , close , close , close , time, volume, isRealtime);
				bars.LastPrice = close;
				return;
			}
			else 
			{
				Data.Bar bar = (Bar) bars.Get(bars.Count - 1); 
				if (bars.Instrument.MasterInstrument.Compare(close, renkoHigh) >= 0) 
				{
					long barVolume = bars.Instrument.MasterInstrument.Compare(close, renkoHigh) == 0 ? volume : 0;
					if (bars.Instrument.MasterInstrument.Compare(bar.Open, renkoHigh - offset) != 0) 
					{
						// direction reversed, change open
						bars.RemoveLastBar();  // Note: bar is now just a local var and not in series!
						AddBar(bars, renkoHigh - offset, renkoHigh, bar.Low, renkoHigh, time, bar.Volume+barVolume, isRealtime);
					}
					else 
						UpdateBar(bars, renkoHigh-offset, renkoHigh, bar.Low, renkoHigh, time, barVolume, isRealtime);

					renkoLow  = renkoHigh - 2.0 * offset;
					renkoHigh = renkoHigh + offset;

					while (bars.Instrument.MasterInstrument.Compare(close, renkoHigh) >= 0)	// add empty bars to fill gap
					{
						barVolume = bars.Instrument.MasterInstrument.Compare(close, renkoHigh) == 0 ? volume : 0;
						AddBar(bars, renkoHigh - offset, renkoHigh, renkoHigh-offset, renkoHigh, time, barVolume, isRealtime);
						renkoLow  = renkoHigh - 2.0 * offset;
						renkoHigh = renkoHigh + offset;
					}

					// add final partial bar
					AddBar(bars, renkoHigh-offset, close, renkoHigh-offset, close, time, barVolume == 0 ? volume : 0, isRealtime);
				}
				else if (bars.Instrument.MasterInstrument.Compare(close, renkoLow) <= 0) 
				{
					long barVolume = bars.Instrument.MasterInstrument.Compare(close, renkoLow) == 0 ? volume : 0;
					if (bars.Instrument.MasterInstrument.Compare(bar.Open, renkoLow + offset) != 0) 
					{
						// direction reversed, change open
						bars.RemoveLastBar();  // Note: bar is now just a local var and not in series!
						AddBar(bars, renkoLow + offset, bar.High, renkoLow, renkoLow, time, bar.Volume+barVolume, isRealtime);
					}
					else
						UpdateBar(bars, renkoLow + offset, bar.High, renkoLow, renkoLow, time, barVolume, isRealtime);

					renkoHigh = renkoLow + 2.0 * offset;
					renkoLow  = renkoLow - offset;

					while (bars.Instrument.MasterInstrument.Compare(close, renkoLow) <= 0)	// add empty bars to fill gap
					{
						barVolume = bars.Instrument.MasterInstrument.Compare(close, renkoLow) == 0 ? volume : 0;
						AddBar(bars, renkoLow + offset, renkoLow+offset, renkoLow, renkoLow, time, barVolume, isRealtime);
						renkoHigh = renkoLow + 2.0 * offset;
						renkoLow  = renkoLow - offset;
					}

					// add final partial bar
					AddBar(bars, renkoLow + offset, renkoLow + offset, close, close, time, barVolume == 0 ? volume : 0, isRealtime);
				}
				else    // Note: open does not really change
					UpdateBar(bars, close, (close > bar.High ? close : bar.High), (close < bar.Low ? close : bar.Low), close, time, volume, isRealtime);

				bars.LastPrice = close;
			}
		}

		public override void ApplyDefaults(Gui.Chart.BarsData barsData)
		{
			barsData.DaysBack		= 3;
			barsData.Period.Value	= 4;
		}

		public override PeriodType BuiltFrom
		{
			get { return PeriodType.Tick; }
		}

		public override string ChartDataBoxDate(DateTime time)
		{
			return time.ToString(Cbi.Globals.CurrentCulture.DateTimeFormat.ShortDatePattern);
		}

		public override string ChartLabel(NinjaTrader.Gui.Chart.ChartControl chartControl, DateTime time)
		{
			return time.ToString(chartControl.LabelFormatSecond, Cbi.Globals.CurrentCulture);
		}

		public override Gui.Chart.ChartStyleType[] ChartStyleTypesSupported
		{
			get { return new Gui.Chart.ChartStyleType[] { Gui.Chart.ChartStyleType.OpenClose, Gui.Chart.ChartStyleType.CandleStick  }; }
		}

		public override object Clone()
		{
			return new WickedRenkoBarsType();
		}

		public override int DefaultValue
		{ 
			get { return 5; }
		}

		public override string DisplayName
		{
			get { return "WickedRenko"; }
		}

		public override int GetInitialLookBackDays(Period period, int barsBack)
		{ 
			return 1;
		}
	
		public override double GetPercentComplete(Data.Bars bars, DateTime now)
		{
			int seconds = bars.Period.Value / 100;

            if (now <= bars.Get(bars.Count - 1).Time)
                return 1.0 - (bars.Get(bars.Count - 1).Time.Subtract(now).TotalSeconds / seconds);
            else
                return 0;
		}

		public override PropertyDescriptorCollection GetProperties(PropertyDescriptor propertyDescriptor, Period period, Attribute[] attributes)
		{
			PropertyDescriptorCollection properties = base.GetProperties(propertyDescriptor, period, attributes);

			// here is how you remove properties not needed for that particular bars type
			properties.Remove(properties.Find("BasePeriodType", true));
			properties.Remove(properties.Find("BasePeriodValue", true));
			properties.Remove(properties.Find("PointAndFigurePriceType", true));
			properties.Remove(properties.Find("ReversalType", true));
			properties.Remove(properties.Find("Value2", true));

			// here is how you change the display name of the property on the properties grid
			Gui.Design.DisplayNameAttribute.SetDisplayName(properties, "Value", "\rBrick size");

			return properties;
		}

		public override bool IsIntraday
		{
			get { return true; }
		}

		public WickedRenkoBarsType() : base(PeriodType.Final2) {	}

		public override string ToString(Period period)
		{
			return String.Format("{0} WickedRenko", period.Value);
		}
	}
}
